/* card.c 
 * io code for the RadioTrack Program 
 * Gideon le Grange, 1994 
 * this is the code that does the actual talking to the card 
 * 1994-11-17 - first success with switching card on and off 
 * 1994-11-18 - put in different tables for on and off   
 * 1994-11-20 - finally figured out how card works 
 *            - no more tables 
 *            - freq converted to bitmap which is sent to the card 
 * 1994-11-22 - removed debugging code. this module is now stable. i hope
 *
 * June 1996 - added code of funcs CardMute and IsTuned 
 *             from xcard.c by Frans Brinkmann
 *
 *             by Thomas Lehmann thole@nuernberg.netsurf.de
 */

#include "port.h"
#include "card.h"
#include <unistd.h>


unsigned short read_card(int sub_port)
{
   /* reads a byte in from the card */
   unsigned short value;
   
   value=port_in(RadioBase+sub_port);
   return(value);
}

void write_card(int sub_port, unsigned short value)
{
   /* writes a byte to the card */
   port_out(value,RadioBase+sub_port);
}

void CardOff(void)
{
   /* switch card off */
   write_card(0,0x00);
   write_card(0,0x00);
}

void VolumeDown(void)
{
   /* vol down a notch */
   write_card(0,0x48);
   usleep(100000);
   write_card(0,0xc8);
}

void VolumeUp(void)
{
   /* vol up a notch */
   write_card(0,0x88);
   usleep(100000);
   write_card(0,0xc8);
}

void Tune(float freq)
{
   /* tunes the radio to the desired frequency */
   unsigned long int bits;
   int i,mask;
   
   bits=(freq*40)+10486188;
   /* yes, i know. now where does THAT come from? 
    * working out that bits=(freq*multiplier)+base 
    * was hard, and working out the base and multiplier
    * was tricky. */
   mask=1;
   for (i=0; i<24; ++i)
     {
	if ((bits&mask)==0)
	  {
	     /* 0 bit */
	     write_card(0,0x1);
	     write_card(0,0x1);
	     write_card(0,0x3);
	     write_card(0,0x3);
	  }
	else
	  {
	     /* bit is 1 */
	     write_card(0,0x5);
	     write_card(0,0x5);
	     write_card(0,0x7);
	     write_card(0,0x7);
	  }
	mask=mask*2;
     }
   /* termination sequence? */
     write_card(0,0x0);
     usleep(10000);
     write_card(0,0xc8);
}

/* CardMute basis from Frans Brinkmann adapted to card.c */

void CardMute(int mute)
{
  if (mute)
  {
    /* shut it up */
    write_card(0,0x0);
    write_card(0,0x0);
  }
  else
  {
    /* turn sound on */
    write_card(0,0x0);
    write_card(0,0xc8);
  }
}

/* IsTuned from Frans Brinkmann adapted to card.c */

int IsTuned(void)
{
int x=0;

/* test tuning */
/*
	DO NOT CHANGE THE TIMING !?!?!?

	Scanning seems to be pretty unreliable, or at least using
	the one card I've got. A shorter wait will miss lots of valid
	stations, longer ones will make it find lots of noisy signals.
        Should anyone ever find out how it is supposed to work please
	let me know. For absolutely no reason that I can see it also seems
        to work slightly better if the frequencies used are multiples
        of .05, but that might just be coincidence
*/
      write_card(0, 0xf8);
      
      usleep(200000);              /* any faster and it won't work? */
      x = (int) read_card(0);
      /* x = inb(RadioBase); */
      usleep(25000);               /* set from 10000 to 25000. is this better? */
      write_card(0,0xe8);

      /* if x == ff not tuned, if x == fd then tuned */

      if (( x & 255) != 0x0ff) return(1);
      else return(0);
}
