/* xradio_cb.c
 * callback functions which handle all actions on pressing
   buttons etc.

   (c) July 1996 by Thomas Lehmann (thole@nuernberg.netsurf.de)
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <forms.h>
#include "priv.h"
#include "card.h"
#include "xradio.h"
#include "station.h"

extern FL_OBJECT *display;
extern FD_xradio *parent;

extern station stations[NumStations];
extern char* myname;
extern float MinFreq;
extern float MaxFreq;
extern float DefaultFreq;

char buffer[13];
int   is_memory;
int   sleep_mode, sleep_set;
double   sleepmin = 59;
int freq_mode;
int akt_station; 

void UpdateDisplay(void)
{
  if (freq_mode)
    sprintf(buffer, "%7.3f MHz", DefaultFreq);
  else
    strcpy(buffer, stations[akt_station].name);
  fl_set_object_label(display, buffer);
}

/* callbacks for form xradio */
void switchpower(FL_OBJECT *ob, long data)
{
  if (fl_get_button_numb(ob) == 3)
    fl_show_message("XRadio", "(c) June 1996 by Thomas Lehmann",
                    "under the terms of the GNU General Public License");

  if (fl_get_button(ob))
  {
    /* init all mode variables */
    is_memory = 0;    /* memory off */
    sleep_mode =0;    /* no sleeping */
    sleep_set = 0;
    freq_mode = 1;    /* freqs are displayed */

    /* ativate all channels and all buttons */
    fl_activate_object(parent->channels);
    fl_activate_object(parent->allbuttons);
        
    /* hide input for new stations */
    fl_hide_object(parent->in_station);
    
    /* tune last used station, if preset turn channel buuton on */
    if (akt_station)
      fl_set_button(parent->c[akt_station], 1);
    DefaultFreq = stations[akt_station].freq;
    Tune(DefaultFreq);
    UpdateDisplay();
  }
  else
  {
    /* turn off all buttons light */
    fl_set_button(parent->bt_mute, 0);
    fl_set_button(parent->bt_mem, 0);
    fl_set_button(parent->bt_sleep, 0);
    if (akt_station)
      fl_set_button(parent->c[akt_station], 0);
    
    /* deactivate all channels and all buttons */
    fl_deactivate_object(parent->channels);
    fl_deactivate_object(parent->allbuttons);
      
    /* hide input for new stations */
    fl_hide_object(parent->in_station);

    StationWrite();
    CardOff();
    fl_set_object_label(display, "");
  }
}

void channel(FL_OBJECT *ob, long data)
{
  /* turn off muting */
  if (fl_get_button(parent->bt_mute))
    fl_set_button(parent->bt_mute, 0);

  if (!is_memory)
  {
    Tune(stations[data].freq);
    DefaultFreq = stations[data].freq;
  }
  else
  {
    if (freq_mode)
      stations[data].freq = DefaultFreq;
    else
      strcpy(stations[data].name, stations[akt_station].name);
    is_memory = 0;
    fl_set_button(parent->bt_mem, 0);
  }
  akt_station = data;
  UpdateDisplay();
}

void volume(FL_OBJECT *ob, long up)
{
  /* turn off muting */
  if (fl_get_button(parent->bt_mute))
    fl_set_button(parent->bt_mute, 0);
        
  if (sleep_set)
  {
    if (up)
      sleepmin++;
    else
      sleepmin--;
    if (sleepmin > 59) sleepmin = 1;
    if (sleepmin < 1) sleepmin = 59;
    sprintf(buffer, "%5.2f min", sleepmin);
    fl_set_object_label(display, buffer);
  }
  else
  {
    if (up)
      VolumeUp();
    else
      VolumeDown();
  }
}

void muting(FL_OBJECT *ob, long data)
{
  if (fl_get_button(ob))
    CardMute(1);
  else
    CardMute(0);
  
}

void finetune(FL_OBJECT *ob, long up)
{
  if (up)
    DefaultFreq = DefaultFreq + 0.025;
  else
    DefaultFreq = DefaultFreq - 0.025;

  if (DefaultFreq < MinFreq) DefaultFreq=MaxFreq;
  if (DefaultFreq > MaxFreq) DefaultFreq=MinFreq;

  Tune(DefaultFreq);
  /* turn off muting */
  if (fl_get_button(parent->bt_mute))
    fl_set_button(parent->bt_mute, 0);
  
  /* turn off channel button */
  if (akt_station)
    fl_set_button(parent->c[akt_station], 0);
 
  akt_station = 0;
  freq_mode = 1;
  UpdateDisplay();
}

void tuning(FL_OBJECT *ob, long up)
{
  int tuned = 0;

  /* turn off muting */
  if (fl_get_button(parent->bt_mute))
    fl_set_button(parent->bt_mute, 0);  
    
  if ( DefaultFreq < MinFreq ) DefaultFreq = MinFreq;
  while (!tuned)
  {
    if (up)
      DefaultFreq = DefaultFreq + 0.05;
    else
      DefaultFreq = DefaultFreq - 0.05;
    if (DefaultFreq > MaxFreq) DefaultFreq = MinFreq;
    if (DefaultFreq < MinFreq) DefaultFreq = MaxFreq;
    Tune(DefaultFreq);
    tuned = IsTuned();
  }
  /* turn off channel button */
  if (akt_station)
    fl_set_button(parent->c[akt_station], 0);
  
  akt_station = 0;
  freq_mode = 1;
  UpdateDisplay();
}

void memory(FL_OBJECT *ob, long data)
{
  /* mem button sets sleeping timer */
  if (sleep_set)
  {
    fl_set_timer(parent->sleep_timer, sleepmin * 60);
    fl_set_button(parent->bt_mem, 0);
    sleep_set = 0; /* setting of timer finished */
    UpdateDisplay();
  }
  else
  {  
    is_memory = fl_get_button(ob); 
    if (is_memory && fl_get_button_numb(ob) == 3)
    {
      /* deactivate channels and all buttons except mem button */
      fl_deactivate_object(parent->channels);
      fl_deactivate_object(parent->allbuttons);
      fl_activate_object(parent->bt_mem);
      /* set default input to current station name , show input */
      fl_set_input(parent->in_station, stations[akt_station].name);
      fl_show_object(parent->in_station);
      fl_hide_object(display);
    }
    else
    {
      /* activate channels and allbuttons , hide input */
      fl_activate_object(parent->channels);
      fl_activate_object(parent->allbuttons);
      fl_show_object(display);
      fl_hide_object(parent->in_station);
    }
  }
}
    
void sleeping(FL_OBJECT *ob, long data)
{
  sleep_mode = fl_get_button(ob);
  if (sleep_mode)
  {
    sleep_set = 1;  /* setting of timer is allowed */
    sprintf(buffer, "%5.2f min", sleepmin);
    fl_set_object_label(display, buffer);
  }
  else
  {
    sleep_set = 0;  /* setting od timer disabled */
    fl_set_timer(parent->sleep_timer, 0); /* timer off */
    UpdateDisplay();
  }
}

void timeroff(FL_OBJECT *ob, long data)
{
  /* sleep button turned off */
  fl_set_button(parent->bt_sleep, 0);
  /* mute button turned on and Card muting */
  fl_set_button(parent->bt_mute, 1);
  CardMute(1);
}

void switchdisplay(FL_OBJECT *ob, long data)
{
  freq_mode = !freq_mode;
  UpdateDisplay();
}

void newstation (FL_OBJECT *ob, long data)
{
  char *newname, *leer;

  /* I don't like typecasting, but I don't no any other way */
  newname = (char *) fl_get_input(ob);
  
  /* check new station name length, replace whitespaces */
  while ((leer = strchr(newname, ' ')) != NULL)
    *leer = '_';
  if (strlen(newname) > 10)
    strncpy(stations[akt_station].name, newname, 10);
  else
    strcpy(stations[akt_station].name, newname);  
  
  fl_show_object(display); 
  fl_hide_object(ob);
  UpdateDisplay();
  is_memory = 0;
  fl_set_button(parent->bt_mem, 0);
  /* activate channels and all buttons */
  fl_activate_object(parent->channels);
  fl_activate_object(parent->allbuttons);
}
